{-# LANGUAGE CPP, TemplateHaskell #-}
-- import FileLocation
#include "consts.h"

main = do
    -- $(undef) -- make sure this also loads TH for a different comparison
  _ERROR("Oh no!")
  _ERROR("Oh no!")
  _ERROR("Oh no!")
  _ERROR("Oh no!")
  _ERROR("Oh no!")
  _ERROR("Oh no!")
  _ERROR("Oh no!")
  _ERROR("Oh no!")
  _ERROR("Oh no!")
  _ERROR("Oh no!")
  _ERROR("Oh no!")
  _ERROR("Oh no!")
  _ERROR("Oh no!")
  _ERROR("Oh no!")
  _ERROR("Oh no!")
  _ERROR("Oh no!")
  _ERROR("Oh no!")
  _ERROR("Oh no!")
  _ERROR("Oh no!")
  _ERROR("Oh no!")
  _ERROR("Oh no!")
  _ERROR("Oh no!")
  _ERROR("Oh no!")
  _ERROR("Oh no!")
  _ERROR("Oh no!")
  _ERROR("Oh no!")
  _ERROR("Oh no!")
  _ERROR("Oh no!")
  _ERROR("Oh no!")
  _ERROR("Oh no!")
